import { Module } from '@nestjs/common';
import { ConfigModule, ConfigService } from '@nestjs/config';
import { TypeOrmModule } from '@nestjs/typeorm';
import { ServeStaticModule } from '@nestjs/serve-static';
import { ScheduleModule } from '@nestjs/schedule';
import { JwtModule } from '@nestjs/jwt';
import { join } from 'path';
import { mkdirSync } from 'fs';

import { AppController } from './app.controller';
import { AppService } from './app.service';
import { AuthModule } from './auth/auth.module';
import { UsersModule } from './users/users.module';
import { ChatModule } from './chat/chat.module';
import { MediaModule } from './media/media.module';
import { NotificationModule } from './notification/notification.module';
import { EncryptionModule } from './encryption/encryption.module';
import { OrganizationModule } from './organization/organization.module';

// Ensure storage directories exist
const storageDir = '/app/storage';
const dbDir = '/app/storage/db';
const mediaDir = '/app/storage/media';

try {
  mkdirSync(storageDir, { recursive: true });
  mkdirSync(dbDir, { recursive: true });
  mkdirSync(mediaDir, { recursive: true });
} catch (error) {
  // Directory already exists or other error
}

@Module({
  imports: [
    ConfigModule.forRoot({
      isGlobal: true,
    }),
    TypeOrmModule.forRootAsync({
      imports: [ConfigModule],
      useFactory: (configService: ConfigService) => ({
        type: 'sqlite',
        database: '/app/storage/db/chat.sqlite',
        entities: [__dirname + '/**/*.entity{.ts,.js}'],
        synchronize: process.env.NODE_ENV !== 'production',
        logging: process.env.NODE_ENV === 'development',
      }),
      inject: [ConfigService],
    }),
    JwtModule.registerAsync({
      imports: [ConfigModule],
      useFactory: async (configService: ConfigService) => ({
        secret: configService.get<string>('JWT_SECRET') || 'your-secret-key',
        signOptions: { expiresIn: '7d' },
      }),
      inject: [ConfigService],
      global: true,
    }),
    ServeStaticModule.forRoot({
      rootPath: '/app/storage/media',
      serveRoot: '/media',
    }),
    ScheduleModule.forRoot(),
    AuthModule,
    UsersModule,
    OrganizationModule,
    ChatModule,
    MediaModule,
    NotificationModule,
    EncryptionModule,
  ],
  controllers: [AppController],
  providers: [AppService],
})
export class AppModule {}