import { Injectable, UnauthorizedException, ConflictException } from '@nestjs/common';
import { JwtService } from '@nestjs/jwt';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';
import * as bcrypt from 'bcrypt';
import { v4 as uuidv4 } from 'uuid';

import { User } from '../database/entities/user.entity';
import { UserDevice } from '../database/entities/user-device.entity';
import { RegisterDto } from './dto/register.dto';
import { LoginDto } from './dto/login.dto';
import { generateKeyPair } from '../encryption/encryption.service';

@Injectable()
export class AuthService {
  constructor(
    @InjectRepository(User)
    private userRepository: Repository<User>,
    @InjectRepository(UserDevice)
    private deviceRepository: Repository<UserDevice>,
    private jwtService: JwtService,
  ) {}

  async validateUser(email: string, password: string): Promise<any> {
    const user = await this.userRepository.findOne({ where: { email } });
    if (user && (await bcrypt.compare(password, user.password))) {
      const { password, ...result } = user;
      return result;
    }
    return null;
  }

  async login(loginDto: LoginDto) {
    const user = await this.validateUser(loginDto.email, loginDto.password);
    if (!user) {
      throw new UnauthorizedException('Invalid credentials');
    }

    await this.userRepository.update(user.id, {
      isOnline: true,
      lastSeen: new Date(),
    });

    if (loginDto.deviceInfo) {
      await this.registerDevice(user.id, loginDto.deviceInfo);
    }

    const payload = { email: user.email, sub: user.id };
    return {
      access_token: this.jwtService.sign(payload),
      user: {
        id: user.id,
        email: user.email,
        username: user.username,
        avatar: user.avatar,
        bio: user.bio,
        publicKey: user.publicKey,
      },
    };
  }

  async register(registerDto: RegisterDto) {
    const existingUser = await this.userRepository.findOne({
      where: [{ email: registerDto.email }, { username: registerDto.username }],
    });

    if (existingUser) {
      throw new ConflictException('User with this email or username already exists');
    }

    const hashedPassword = await bcrypt.hash(registerDto.password, 12);
    const keyPair = generateKeyPair();

    const user = this.userRepository.create({
      email: registerDto.email,
      username: registerDto.username,
      password: hashedPassword,
      publicKey: keyPair.publicKey,
      privateKey: keyPair.privateKey,
      isOnline: true,
    });

    const savedUser = await this.userRepository.save(user);

    if (registerDto.deviceInfo) {
      await this.registerDevice(savedUser.id, registerDto.deviceInfo);
    }

    const payload = { email: savedUser.email, sub: savedUser.id };
    return {
      access_token: this.jwtService.sign(payload),
      user: {
        id: savedUser.id,
        email: savedUser.email,
        username: savedUser.username,
        avatar: savedUser.avatar,
        bio: savedUser.bio,
        publicKey: savedUser.publicKey,
      },
    };
  }

  async logout(userId: string, deviceId?: string) {
    await this.userRepository.update(userId, {
      isOnline: false,
      lastSeen: new Date(),
    });

    if (deviceId) {
      await this.deviceRepository.update(
        { user: { id: userId }, deviceId },
        { isActive: false }
      );
    }

    return { message: 'Logged out successfully' };
  }

  private async registerDevice(userId: string, deviceInfo: any) {
    const existingDevice = await this.deviceRepository.findOne({
      where: { user: { id: userId }, deviceId: deviceInfo.deviceId },
    });

    if (existingDevice) {
      await this.deviceRepository.update(existingDevice.id, {
        fcmToken: deviceInfo.fcmToken,
        isActive: true,
      });
    } else {
      const device = this.deviceRepository.create({
        deviceId: deviceInfo.deviceId,
        deviceType: deviceInfo.deviceType,
        platform: deviceInfo.platform,
        fcmToken: deviceInfo.fcmToken,
        user: { id: userId },
      });
      await this.deviceRepository.save(device);
    }
  }

  async findUserById(id: string): Promise<User> {
    return this.userRepository.findOne({ where: { id } });
  }
}