import {
  Controller,
  Get,
  Post,
  Put,
  Delete,
  Body,
  Param,
  Query,
  UseGuards,
  Request,
} from '@nestjs/common';
import { ApiTags, ApiOperation, ApiResponse, ApiBearerAuth, ApiHeader } from '@nestjs/swagger';
import { ChatService } from './chat.service';
import { CreateChatRoomDto } from './dto/create-chat-room.dto';
import { CreateMessageDto } from './dto/create-message.dto';
import { JwtAuthGuard } from '../auth/guards/jwt-auth.guard';
import { OrganizationGuard } from '../organization/guards/organization.guard';

@ApiTags('Chat')
@Controller('chat')
@UseGuards(JwtAuthGuard, OrganizationGuard)
@ApiBearerAuth()
@ApiHeader({
  name: 'x-organization-id',
  description: 'Organization ID for multi-tenant access',
  required: false,
})
export class ChatController {
  constructor(private readonly chatService: ChatService) {}

  @Get('rooms')
  @ApiOperation({ summary: 'Get user chat rooms in organization' })
  @ApiResponse({ status: 200, description: 'Chat rooms retrieved successfully' })
  async getUserChatRooms(@Request() req) {
    return this.chatService.getUserChatRooms(req.user.id, req.organization.id);
  }

  @Post('rooms/direct')
  @ApiOperation({ summary: 'Create or get direct message chat room' })
  @ApiResponse({ status: 201, description: 'Direct message room created or retrieved' })
  async createDirectMessage(
    @Request() req,
    @Body('recipientId') recipientId: string,
  ) {
    return this.chatService.createDirectMessage(req.user.id, recipientId, req.organization.id);
  }

  @Post('rooms/group')
  @ApiOperation({ summary: 'Create group chat room' })
  @ApiResponse({ status: 201, description: 'Group chat room created successfully' })
  async createGroupChat(
    @Request() req,
    @Body() createChatRoomDto: CreateChatRoomDto,
  ) {
    return this.chatService.createGroupChat(createChatRoomDto, req.user.id, req.organization.id);
  }

  @Get('rooms/:roomId')
  @ApiOperation({ summary: 'Get chat room details' })
  @ApiResponse({ status: 200, description: 'Chat room retrieved successfully' })
  async getChatRoom(@Param('roomId') roomId: string, @Request() req) {
    return this.chatService.getChatRoom(roomId, req.user.id, req.organization.id);
  }

  @Get('rooms/:roomId/messages')
  @ApiOperation({ summary: 'Get chat room messages' })
  @ApiResponse({ status: 200, description: 'Messages retrieved successfully' })
  async getChatRoomMessages(
    @Param('roomId') roomId: string,
    @Request() req,
    @Query('page') page?: number,
    @Query('limit') limit?: number,
  ) {
    return this.chatService.getChatRoomMessages(roomId, req.user.id, req.organization.id, page, limit);
  }

  @Post('messages')
  @ApiOperation({ summary: 'Create a new message' })
  @ApiResponse({ status: 201, description: 'Message created successfully' })
  async createMessage(@Request() req, @Body() createMessageDto: CreateMessageDto) {
    return this.chatService.createMessage({
      ...createMessageDto,
      senderId: req.user.id,
      organizationId: req.organization.id,
    });
  }

  @Put('messages/:messageId')
  @ApiOperation({ summary: 'Edit message' })
  @ApiResponse({ status: 200, description: 'Message updated successfully' })
  async editMessage(
    @Param('messageId') messageId: string,
    @Request() req,
    @Body('content') content: string,
  ) {
    return this.chatService.editMessage(messageId, content, req.user.id, req.organization.id);
  }

  @Delete('messages/:messageId')
  @ApiOperation({ summary: 'Delete message' })
  @ApiResponse({ status: 200, description: 'Message deleted successfully' })
  async deleteMessage(@Param('messageId') messageId: string, @Request() req) {
    return this.chatService.deleteMessage(messageId, req.user.id, req.organization.id);
  }

  @Post('rooms/:roomId/members')
  @ApiOperation({ summary: 'Add members to group chat' })
  @ApiResponse({ status: 200, description: 'Members added successfully' })
  async addMembersToGroup(
    @Param('roomId') roomId: string,
    @Request() req,
    @Body('memberIds') memberIds: string[],
  ) {
    return this.chatService.addMemberToGroup(roomId, memberIds, req.user.id, req.organization.id);
  }

  @Delete('rooms/:roomId/members/:memberId')
  @ApiOperation({ summary: 'Remove member from group chat' })
  @ApiResponse({ status: 200, description: 'Member removed successfully' })
  async removeMemberFromGroup(
    @Param('roomId') roomId: string,
    @Param('memberId') memberId: string,
    @Request() req,
  ) {
    return this.chatService.removeMemberFromGroup(roomId, memberId, req.user.id, req.organization.id);
  }
}