import {
  WebSocketGateway,
  SubscribeMessage,
  MessageBody,
  ConnectedSocket,
  OnGatewayConnection,
  OnGatewayDisconnect,
  WebSocketServer,
} from '@nestjs/websockets';
import { Server, Socket } from 'socket.io';
import { UseGuards } from '@nestjs/common';
import { JwtService } from '@nestjs/jwt';
import { ChatService } from './chat.service';
import { UsersService } from '../users/users.service';
import { NotificationService } from '../notification/notification.service';
import { OrganizationService } from '../organization/organization.service';

interface AuthenticatedSocket extends Socket {
  user: {
    id: string;
    email: string;
    username: string;
  };
  organization: {
    id: string;
    name: string;
    slug: string;
  };
}

@WebSocketGateway({
  cors: {
    origin: '*',
  },
})
export class ChatGateway implements OnGatewayConnection, OnGatewayDisconnect {
  @WebSocketServer()
  server: Server;

  private connectedUsers = new Map<string, { socketId: string; organizationId: string }>(); // userId -> {socketId, organizationId}
  private organizationRooms = new Map<string, Set<string>>(); // organizationId -> Set of socketIds

  constructor(
    private chatService: ChatService,
    private usersService: UsersService,
    private notificationService: NotificationService,
    private organizationService: OrganizationService,
    private jwtService: JwtService,
  ) {}

  async handleConnection(client: Socket) {
    try {
      const token = client.handshake.auth.token || client.handshake.headers.authorization?.split(' ')[1];
      const organizationId = client.handshake.auth.organizationId || client.handshake.headers['x-organization-id'];
      
      if (!token) {
        client.disconnect();
        return;
      }

      const payload = this.jwtService.verify(token);
      const user = await this.usersService.findOne(payload.sub);
      
      if (!user) {
        client.disconnect();
        return;
      }

      // Get user's active organization if not provided
      let activeOrgId = organizationId;
      if (!activeOrgId) {
        const activeOrg = await this.organizationService.getUserActiveOrganization(user.id);
        activeOrgId = activeOrg?.id;
      }

      if (!activeOrgId) {
        client.emit('error', { message: 'No organization context provided' });
        client.disconnect();
        return;
      }

      // Verify user is member of the organization
      const isMember = await this.organizationService.isUserMember(activeOrgId, user.id);
      if (!isMember) {
        client.emit('error', { message: 'You are not a member of this organization' });
        client.disconnect();
        return;
      }

      const organization = await this.organizationService.getOrganization(activeOrgId, user.id);

      (client as AuthenticatedSocket).user = {
        id: user.id,
        email: user.email,
        username: user.username,
      };

      (client as AuthenticatedSocket).organization = {
        id: organization.id,
        name: organization.name,
        slug: organization.slug,
      };

      // Store connection info
      this.connectedUsers.set(user.id, { socketId: client.id, organizationId: activeOrgId });
      
      // Add to organization room
      if (!this.organizationRooms.has(activeOrgId)) {
        this.organizationRooms.set(activeOrgId, new Set());
      }
      this.organizationRooms.get(activeOrgId).add(client.id);
      client.join(`org_${activeOrgId}`);

      await this.usersService.updateOnlineStatus(user.id, true);

      client.emit('connected', { 
        message: 'Connected successfully', 
        user: (client as AuthenticatedSocket).user,
        organization: (client as AuthenticatedSocket).organization,
      });
      
      // Join user to their chat rooms within the organization
      const chatRooms = await this.chatService.getUserChatRooms(user.id, activeOrgId);
      chatRooms.forEach(room => {
        client.join(`room_${room.id}`);
      });

      // Notify other users in the same organization that this user is online
      client.to(`org_${activeOrgId}`).emit('user_online', { 
        userId: user.id, 
        username: user.username,
        organizationId: activeOrgId,
      });
      
    } catch (error) {
      client.emit('error', { message: 'Authentication failed' });
      client.disconnect();
    }
  }

  async handleDisconnect(client: AuthenticatedSocket) {
    if (client.user && client.organization) {
      const connectionInfo = this.connectedUsers.get(client.user.id);
      
      if (connectionInfo) {
        this.connectedUsers.delete(client.user.id);
        
        // Remove from organization room
        const orgSockets = this.organizationRooms.get(client.organization.id);
        if (orgSockets) {
          orgSockets.delete(client.id);
          if (orgSockets.size === 0) {
            this.organizationRooms.delete(client.organization.id);
          }
        }
      }

      await this.usersService.updateOnlineStatus(client.user.id, false);
      
      // Notify other users in the same organization that this user is offline
      client.to(`org_${client.organization.id}`).emit('user_offline', { 
        userId: client.user.id, 
        username: client.user.username,
        organizationId: client.organization.id,
      });
    }
  }

  @SubscribeMessage('join_room')
  async handleJoinRoom(
    @MessageBody() data: { roomId: string },
    @ConnectedSocket() client: AuthenticatedSocket,
  ) {
    try {
      const room = await this.chatService.getChatRoom(data.roomId, client.user.id, client.organization.id);
      client.join(`room_${data.roomId}`);
      client.emit('joined_room', { roomId: data.roomId, room });
    } catch (error) {
      client.emit('error', { message: error.message });
    }
  }

  @SubscribeMessage('leave_room')
  async handleLeaveRoom(
    @MessageBody() data: { roomId: string },
    @ConnectedSocket() client: AuthenticatedSocket,
  ) {
    client.leave(`room_${data.roomId}`);
    client.emit('left_room', { roomId: data.roomId });
  }

  @SubscribeMessage('send_message')
  async handleMessage(
    @MessageBody() data: { roomId: string; content: string; type?: string; replyToId?: string },
    @ConnectedSocket() client: AuthenticatedSocket,
  ) {
    try {
      const message = await this.chatService.createMessage({
        content: data.content,
        type: data.type || 'text',
        senderId: client.user.id,
        chatRoomId: data.roomId,
        replyToId: data.replyToId,
        organizationId: client.organization.id,
      });

      const populatedMessage = await this.chatService.getMessage(message.id, client.organization.id);
      
      // Send message to all users in the room
      this.server.to(`room_${data.roomId}`).emit('new_message', populatedMessage);

      // Handle mentions
      await this.handleMentions(populatedMessage, client.user.id, client.organization.id);

      // Send push notifications to offline users
      await this.notificationService.sendNewMessageNotifications(populatedMessage);

    } catch (error) {
      client.emit('error', { message: error.message });
    }
  }

  @SubscribeMessage('edit_message')
  async handleEditMessage(
    @MessageBody() data: { messageId: string; content: string },
    @ConnectedSocket() client: AuthenticatedSocket,
  ) {
    try {
      const message = await this.chatService.editMessage(
        data.messageId, 
        data.content, 
        client.user.id, 
        client.organization.id
      );
      this.server.to(`room_${message.chatRoom.id}`).emit('message_edited', message);
    } catch (error) {
      client.emit('error', { message: error.message });
    }
  }

  @SubscribeMessage('delete_message')
  async handleDeleteMessage(
    @MessageBody() data: { messageId: string },
    @ConnectedSocket() client: AuthenticatedSocket,
  ) {
    try {
      const message = await this.chatService.deleteMessage(
        data.messageId, 
        client.user.id, 
        client.organization.id
      );
      this.server.to(`room_${message.chatRoom.id}`).emit('message_deleted', { messageId: data.messageId });
    } catch (error) {
      client.emit('error', { message: error.message });
    }
  }

  @SubscribeMessage('typing_start')
  async handleTypingStart(
    @MessageBody() data: { roomId: string },
    @ConnectedSocket() client: AuthenticatedSocket,
  ) {
    client.to(`room_${data.roomId}`).emit('user_typing', {
      userId: client.user.id,
      username: client.user.username,
      roomId: data.roomId,
    });
  }

  @SubscribeMessage('typing_stop')
  async handleTypingStop(
    @MessageBody() data: { roomId: string },
    @ConnectedSocket() client: AuthenticatedSocket,
  ) {
    client.to(`room_${data.roomId}`).emit('user_stopped_typing', {
      userId: client.user.id,
      username: client.user.username,
      roomId: data.roomId,
    });
  }

  @SubscribeMessage('mark_as_read')
  async handleMarkAsRead(
    @MessageBody() data: { roomId: string; messageId: string },
    @ConnectedSocket() client: AuthenticatedSocket,
  ) {
    // In a full implementation, you would track read receipts
    client.to(`room_${data.roomId}`).emit('message_read', {
      messageId: data.messageId,
      userId: client.user.id,
      roomId: data.roomId,
    });
  }

  @SubscribeMessage('switch_organization')
  async handleSwitchOrganization(
    @MessageBody() data: { organizationId: string },
    @ConnectedSocket() client: AuthenticatedSocket,
  ) {
    try {
      // Verify user is member of the new organization
      const isMember = await this.organizationService.isUserMember(data.organizationId, client.user.id);
      if (!isMember) {
        client.emit('error', { message: 'You are not a member of this organization' });
        return;
      }

      // Leave current organization room and chat rooms
      client.leave(`org_${client.organization.id}`);
      const currentChatRooms = await this.chatService.getUserChatRooms(client.user.id, client.organization.id);
      currentChatRooms.forEach(room => {
        client.leave(`room_${room.id}`);
      });

      // Update organization context
      const newOrganization = await this.organizationService.getOrganization(data.organizationId, client.user.id);
      (client as AuthenticatedSocket).organization = {
        id: newOrganization.id,
        name: newOrganization.name,
        slug: newOrganization.slug,
      };

      // Join new organization room and chat rooms
      client.join(`org_${data.organizationId}`);
      const newChatRooms = await this.chatService.getUserChatRooms(client.user.id, data.organizationId);
      newChatRooms.forEach(room => {
        client.join(`room_${room.id}`);
      });

      // Update connection tracking
      this.connectedUsers.set(client.user.id, { socketId: client.id, organizationId: data.organizationId });

      // Set as active organization
      await this.organizationService.setUserActiveOrganization(client.user.id, data.organizationId);

      client.emit('organization_switched', { 
        organization: client.organization,
        chatRooms: newChatRooms,
      });

    } catch (error) {
      client.emit('error', { message: error.message });
    }
  }

  private async handleMentions(message: any, senderId: string, organizationId: string) {
    if (message.mentions && message.mentions.length > 0) {
      for (const mention of message.mentions) {
        const mentionedUserConnection = this.connectedUsers.get(mention.mentionedUser.id);
        
        // Only send mention if user is in the same organization
        if (mentionedUserConnection && mentionedUserConnection.organizationId === organizationId) {
          this.server.to(mentionedUserConnection.socketId).emit('mentioned', {
            message,
            mentionedBy: senderId,
            organizationId,
          });
        }

        // Send push notification for mention
        await this.notificationService.sendMentionNotification(message, mention.mentionedUser.id);
      }
    }
  }

  // Method to send message to specific user in specific organization
  sendToUserInOrganization(userId: string, organizationId: string, event: string, data: any) {
    const connection = this.connectedUsers.get(userId);
    if (connection && connection.organizationId === organizationId) {
      this.server.to(connection.socketId).emit(event, data);
    }
  }

  // Method to send message to room
  sendToRoom(roomId: string, event: string, data: any) {
    this.server.to(`room_${roomId}`).emit(event, data);
  }

  // Method to send message to organization
  sendToOrganization(organizationId: string, event: string, data: any) {
    this.server.to(`org_${organizationId}`).emit(event, data);
  }
}