import { Injectable, NotFoundException, ForbiddenException, BadRequestException } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository, In } from 'typeorm';
import { Message, MessageType } from '../database/entities/message.entity';
import { ChatRoom, ChatRoomType } from '../database/entities/chat-room.entity';
import { MessageMention } from '../database/entities/message-mention.entity';
import { User } from '../database/entities/user.entity';
import { Organization } from '../database/entities/organization.entity';
import { EncryptionService } from '../encryption/encryption.service';
import { CreateMessageDto } from './dto/create-message.dto';
import { CreateChatRoomDto } from './dto/create-chat-room.dto';

@Injectable()
export class ChatService {
  constructor(
    @InjectRepository(Message)
    private messageRepository: Repository<Message>,
    @InjectRepository(ChatRoom)
    private chatRoomRepository: Repository<ChatRoom>,
    @InjectRepository(MessageMention)
    private mentionRepository: Repository<MessageMention>,
    @InjectRepository(User)
    private userRepository: Repository<User>,
    @InjectRepository(Organization)
    private organizationRepository: Repository<Organization>,
    private encryptionService: EncryptionService,
  ) {}

  async createDirectMessage(senderId: string, recipientId: string, organizationId: string): Promise<ChatRoom> {
    // Verify organization
    const organization = await this.organizationRepository.findOne({ where: { id: organizationId } });
    if (!organization) {
      throw new NotFoundException('Organization not found');
    }

    // Check if DM already exists between these users in this organization
    const existingDM = await this.chatRoomRepository
      .createQueryBuilder('room')
      .innerJoin('room.members', 'member')
      .where('room.type = :type', { type: ChatRoomType.DIRECT })
      .andWhere('room.organizationId = :organizationId', { organizationId })
      .groupBy('room.id')
      .having('COUNT(member.id) = 2')
      .andHaving('SUM(CASE WHEN member.id IN (:...userIds) THEN 1 ELSE 0 END) = 2', {
        userIds: [senderId, recipientId],
      })
      .getOne();

    if (existingDM) {
      return this.getChatRoom(existingDM.id, senderId, organizationId);
    }

    // Create new DM
    const users = await this.userRepository.findByIds([senderId, recipientId]);
    if (users.length !== 2) {
      throw new NotFoundException('One or both users not found');
    }

    const chatRoom = this.chatRoomRepository.create({
      type: ChatRoomType.DIRECT,
      members: users,
      organization,
      isEncrypted: true,
      encryptionKey: this.encryptionService.generateChatRoomKey(),
    });

    return this.chatRoomRepository.save(chatRoom);
  }

  async createGroupChat(createChatRoomDto: CreateChatRoomDto, creatorId: string, organizationId: string): Promise<ChatRoom> {
    const { name, description, memberIds } = createChatRoomDto;
    
    // Verify organization
    const organization = await this.organizationRepository.findOne({ where: { id: organizationId } });
    if (!organization) {
      throw new NotFoundException('Organization not found');
    }
    
    const allMemberIds = [...new Set([creatorId, ...memberIds])];
    const members = await this.userRepository.findByIds(allMemberIds);
    
    if (members.length !== allMemberIds.length) {
      throw new NotFoundException('One or more users not found');
    }

    const creator = members.find(user => user.id === creatorId);
    
    const chatRoom = this.chatRoomRepository.create({
      name,
      description,
      type: ChatRoomType.GROUP,
      members,
      organization,
      createdBy: creator,
      isEncrypted: true,
      encryptionKey: this.encryptionService.generateChatRoomKey(),
    });

    return this.chatRoomRepository.save(chatRoom);
  }

  async getChatRoom(roomId: string, userId: string, organizationId?: string): Promise<ChatRoom> {
    const queryBuilder = this.chatRoomRepository
      .createQueryBuilder('room')
      .leftJoinAndSelect('room.members', 'members')
      .leftJoinAndSelect('room.createdBy', 'createdBy')
      .leftJoinAndSelect('room.organization', 'organization')
      .where('room.id = :roomId', { roomId });

    if (organizationId) {
      queryBuilder.andWhere('room.organizationId = :organizationId', { organizationId });
    }

    const chatRoom = await queryBuilder.getOne();

    if (!chatRoom) {
      throw new NotFoundException('Chat room not found');
    }

    // Check if user is a member of the chat room
    const isMember = chatRoom.members.some(member => member.id === userId);
    if (!isMember) {
      throw new ForbiddenException('You are not a member of this chat room');
    }

    return chatRoom;
  }

  async getUserChatRooms(userId: string, organizationId: string): Promise<ChatRoom[]> {
    return this.chatRoomRepository
      .createQueryBuilder('room')
      .innerJoin('room.members', 'member', 'member.id = :userId', { userId })
      .leftJoinAndSelect('room.members', 'allMembers')
      .leftJoinAndSelect('room.createdBy', 'creator')
      .leftJoinAndSelect('room.organization', 'organization')
      .where('room.organizationId = :organizationId', { organizationId })
      .orderBy('room.updatedAt', 'DESC')
      .getMany();
  }

  async createMessage(createMessageDto: CreateMessageDto & { organizationId: string }): Promise<Message> {
    const { content, type, senderId, chatRoomId, replyToId, organizationId } = createMessageDto;

    // Verify user is member of chat room and organization
    const chatRoom = await this.getChatRoom(chatRoomId, senderId, organizationId);
    const sender = await this.userRepository.findOne({ where: { id: senderId } });
    const organization = await this.organizationRepository.findOne({ where: { id: organizationId } });

    let replyTo = null;
    if (replyToId) {
      replyTo = await this.messageRepository.findOne({
        where: { id: replyToId, chatRoom: { id: chatRoomId }, organization: { id: organizationId } },
      });
      if (!replyTo) {
        throw new NotFoundException('Reply message not found');
      }
    }

    // Encrypt message if chat room is encrypted
    let encryptedContent = null;
    if (chatRoom.isEncrypted) {
      encryptedContent = this.encryptionService.encryptMessage(content, chatRoom.encryptionKey);
    }

    const message = this.messageRepository.create({
      content,
      type: type as MessageType,
      sender,
      chatRoom,
      organization,
      replyTo,
      isEncrypted: chatRoom.isEncrypted,
      encryptedContent,
    });

    const savedMessage = await this.messageRepository.save(message);

    // Handle mentions
    await this.handleMentions(content, savedMessage, organizationId);

    // Update chat room's updatedAt
    await this.chatRoomRepository.update(chatRoomId, { updatedAt: new Date() });

    return savedMessage;
  }

  async getMessage(messageId: string, organizationId: string): Promise<Message> {
    const message = await this.messageRepository.findOne({
      where: { id: messageId, organization: { id: organizationId } },
      relations: ['sender', 'chatRoom', 'organization', 'replyTo', 'replyTo.sender', 'mentions', 'mentions.mentionedUser', 'media'],
    });

    if (!message) {
      throw new NotFoundException('Message not found');
    }

    // Decrypt message if encrypted
    if (message.isEncrypted && message.encryptedContent) {
      const chatRoom = await this.chatRoomRepository.findOne({ where: { id: message.chatRoom.id } });
      message.content = this.encryptionService.decryptMessage(message.encryptedContent, chatRoom.encryptionKey);
    }

    return message;
  }

  async getChatRoomMessages(
    roomId: string,
    userId: string,
    organizationId: string,
    page = 1,
    limit = 50,
  ): Promise<{ messages: Message[]; total: number }> {
    // Verify user access to chat room
    await this.getChatRoom(roomId, userId, organizationId);

    const skip = (page - 1) * limit;

    const [messages, total] = await this.messageRepository.findAndCount({
      where: { 
        chatRoom: { id: roomId }, 
        organization: { id: organizationId },
        isDeleted: false 
      },
      relations: ['sender', 'replyTo', 'replyTo.sender', 'mentions', 'mentions.mentionedUser', 'media'],
      order: { createdAt: 'DESC' },
      skip,
      take: limit,
    });

    // Decrypt encrypted messages
    const chatRoom = await this.chatRoomRepository.findOne({ where: { id: roomId } });
    if (chatRoom.isEncrypted) {
      messages.forEach(message => {
        if (message.encryptedContent) {
          message.content = this.encryptionService.decryptMessage(message.encryptedContent, chatRoom.encryptionKey);
        }
      });
    }

    return { messages: messages.reverse(), total };
  }

  async editMessage(messageId: string, newContent: string, userId: string, organizationId: string): Promise<Message> {
    const message = await this.messageRepository.findOne({
      where: { id: messageId, organization: { id: organizationId } },
      relations: ['sender', 'chatRoom'],
    });

    if (!message) {
      throw new NotFoundException('Message not found');
    }

    if (message.sender.id !== userId) {
      throw new ForbiddenException('You can only edit your own messages');
    }

    // Encrypt new content if needed
    let encryptedContent = null;
    if (message.isEncrypted) {
      const chatRoom = await this.chatRoomRepository.findOne({ where: { id: message.chatRoom.id } });
      encryptedContent = this.encryptionService.encryptMessage(newContent, chatRoom.encryptionKey);
    }

    await this.messageRepository.update(messageId, {
      content: newContent,
      encryptedContent,
      isEdited: true,
      editedAt: new Date(),
    });

    return this.getMessage(messageId, organizationId);
  }

  async deleteMessage(messageId: string, userId: string, organizationId: string): Promise<Message> {
    const message = await this.messageRepository.findOne({
      where: { id: messageId, organization: { id: organizationId } },
      relations: ['sender', 'chatRoom'],
    });

    if (!message) {
      throw new NotFoundException('Message not found');
    }

    if (message.sender.id !== userId) {
      throw new ForbiddenException('You can only delete your own messages');
    }

    await this.messageRepository.update(messageId, {
      isDeleted: true,
      content: 'This message was deleted',
      encryptedContent: null,
    });

    return message;
  }

  async addMemberToGroup(roomId: string, memberIds: string[], userId: string, organizationId: string): Promise<ChatRoom> {
    const chatRoom = await this.getChatRoom(roomId, userId, organizationId);
    
    if (chatRoom.type !== ChatRoomType.GROUP) {
      throw new BadRequestException('Can only add members to group chats');
    }

    // Check if user is creator or admin (for simplicity, only creator can add members)
    if (chatRoom.createdBy.id !== userId) {
      throw new ForbiddenException('Only group creator can add members');
    }

    const newMembers = await this.userRepository.findByIds(memberIds);
    if (newMembers.length !== memberIds.length) {
      throw new NotFoundException('One or more users not found');
    }

    // Add new members to existing members
    const existingMemberIds = chatRoom.members.map(member => member.id);
    const membersToAdd = newMembers.filter(member => !existingMemberIds.includes(member.id));
    
    chatRoom.members = [...chatRoom.members, ...membersToAdd];
    
    return this.chatRoomRepository.save(chatRoom);
  }

  async removeMemberFromGroup(roomId: string, memberIdToRemove: string, userId: string, organizationId: string): Promise<ChatRoom> {
    const chatRoom = await this.getChatRoom(roomId, userId, organizationId);
    
    if (chatRoom.type !== ChatRoomType.GROUP) {
      throw new BadRequestException('Can only remove members from group chats');
    }

    // Check if user is creator or the member being removed
    if (chatRoom.createdBy.id !== userId && memberIdToRemove !== userId) {
      throw new ForbiddenException('Only group creator can remove members or users can leave themselves');
    }

    chatRoom.members = chatRoom.members.filter(member => member.id !== memberIdToRemove);
    
    return this.chatRoomRepository.save(chatRoom);
  }

  private async handleMentions(content: string, message: Message, organizationId: string) {
    // Extract mentions from content (e.g., @username or @userId)
    const mentionRegex = /@(\w+)/g;
    const mentions = content.match(mentionRegex);

    if (mentions) {
      const usernames = mentions.map(mention => mention.substring(1));
      
      // Find users in the same organization
      const mentionedUsers = await this.userRepository
        .createQueryBuilder('user')
        .innerJoin('user.organizationMemberships', 'membership')
        .where('user.username IN (:...usernames)', { usernames })
        .andWhere('membership.organizationId = :organizationId', { organizationId })
        .andWhere('membership.status = :status', { status: 'active' })
        .getMany();

      for (const user of mentionedUsers) {
        const mention = this.mentionRepository.create({
          message,
          mentionedUser: user,
        });
        await this.mentionRepository.save(mention);
      }
    }
  }
}