import { Injectable, Logger } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';
import { Cron, CronExpression } from '@nestjs/schedule';
import { ConfigService } from '@nestjs/config';
import * as admin from 'firebase-admin';
import { Notification, NotificationType } from '../database/entities/notification.entity';
import { User } from '../database/entities/user.entity';
import { UserDevice } from '../database/entities/user-device.entity';
import { Message } from '../database/entities/message.entity';

@Injectable()
export class NotificationService {
  private readonly logger = new Logger(NotificationService.name);
  private firebaseApp: admin.app.App;

  constructor(
    @InjectRepository(Notification)
    private notificationRepository: Repository<Notification>,
    @InjectRepository(User)
    private userRepository: Repository<User>,
    @InjectRepository(UserDevice)
    private deviceRepository: Repository<UserDevice>,
    @InjectRepository(Message)
    private messageRepository: Repository<Message>,
    private configService: ConfigService,
  ) {
    this.initializeFirebase();
  }

  private initializeFirebase() {
    try {
      const serviceAccount = this.configService.get('FIREBASE_SERVICE_ACCOUNT');
      if (serviceAccount) {
        this.firebaseApp = admin.initializeApp({
          credential: admin.credential.cert(JSON.parse(serviceAccount)),
        });
        this.logger.log('Firebase Admin initialized successfully');
      } else {
        this.logger.warn('Firebase service account not configured. Push notifications will not work.');
      }
    } catch (error) {
      this.logger.error('Failed to initialize Firebase Admin', error);
    }
  }

  async createNotification(
    recipientId: string,
    type: NotificationType,
    title: string,
    body: string,
    data?: any,
    messageId?: string,
  ): Promise<Notification> {
    const recipient = await this.userRepository.findOne({ where: { id: recipientId } });
    const message = messageId ? await this.messageRepository.findOne({ where: { id: messageId } }) : null;

    const notification = this.notificationRepository.create({
      recipient,
      type,
      title,
      body,
      data,
      message,
    });

    return this.notificationRepository.save(notification);
  }

  async sendNewMessageNotifications(message: any): Promise<void> {
    try {
      // Get all members of the chat room except the sender
      const recipients = message.chatRoom.members.filter(
        (member: any) => member.id !== message.sender.id && !member.isOnline
      );

      for (const recipient of recipients) {
        const title = message.chatRoom.type === 'direct' 
          ? `New message from ${message.sender.username}`
          : `${message.sender.username} in ${message.chatRoom.name}`;

        const body = message.type === 'text' 
          ? message.content.substring(0, 100)
          : `Sent a ${message.type}`;

        const notification = await this.createNotification(
          recipient.id,
          NotificationType.MESSAGE,
          title,
          body,
          {
            chatRoomId: message.chatRoom.id,
            senderId: message.sender.id,
            messageType: message.type,
          },
          message.id,
        );

        await this.sendPushNotification(notification);
      }
    } catch (error) {
      this.logger.error('Failed to send new message notifications', error);
    }
  }

  async sendMentionNotification(message: any, mentionedUserId: string): Promise<void> {
    try {
      const mentionedUser = await this.userRepository.findOne({ where: { id: mentionedUserId } });
      
      if (!mentionedUser || mentionedUser.isOnline) {
        return; // Don't send push notification if user is online
      }

      const title = `You were mentioned by ${message.sender.username}`;
      const body = message.content.substring(0, 100);

      const notification = await this.createNotification(
        mentionedUserId,
        NotificationType.MENTION,
        title,
        body,
        {
          chatRoomId: message.chatRoom.id,
          senderId: message.sender.id,
          messageId: message.id,
        },
        message.id,
      );

      await this.sendPushNotification(notification);
    } catch (error) {
      this.logger.error('Failed to send mention notification', error);
    }
  }

  async sendPushNotification(notification: Notification): Promise<void> {
    if (!this.firebaseApp) {
      this.logger.warn('Firebase not initialized. Cannot send push notification.');
      return;
    }

    try {
      const devices = await this.deviceRepository.find({
        where: { user: { id: notification.recipient.id }, isActive: true },
      });

      if (devices.length === 0) {
        this.logger.log(`No active devices found for user ${notification.recipient.id}`);
        return;
      }

      const tokens = devices
        .filter(device => device.fcmToken)
        .map(device => device.fcmToken);

      if (tokens.length === 0) {
        this.logger.log(`No FCM tokens found for user ${notification.recipient.id}`);
        return;
      }

      const message = {
        notification: {
          title: notification.title,
          body: notification.body,
        },
        data: {
          notificationId: notification.id,
          type: notification.type,
          ...notification.data,
        },
        tokens,
      };

      const response = await admin.messaging().sendMulticast(message);
      
      this.logger.log(`Push notification sent successfully. Success: ${response.successCount}, Failure: ${response.failureCount}`);

      // Mark notification as sent
      await this.notificationRepository.update(notification.id, { isSent: true });

      // Handle failed tokens (optional - remove invalid tokens)
      if (response.failureCount > 0) {
        const failedTokens: string[] = [];
        response.responses.forEach((resp, idx) => {
          if (!resp.success) {
            failedTokens.push(tokens[idx]);
          }
        });

        // Remove invalid tokens
        await this.deviceRepository.update(
          { fcmToken: { $in: failedTokens } as any },
          { fcmToken: null }
        );
      }
    } catch (error) {
      this.logger.error('Failed to send push notification', error);
    }
  }

  async getUserNotifications(
    userId: string,
    page = 1,
    limit = 20,
  ): Promise<{ notifications: Notification[]; total: number }> {
    const skip = (page - 1) * limit;

    const [notifications, total] = await this.notificationRepository.findAndCount({
      where: { recipient: { id: userId } },
      relations: ['message', 'message.sender', 'message.chatRoom'],
      order: { createdAt: 'DESC' },
      skip,
      take: limit,
    });

    return { notifications, total };
  }

  async markNotificationAsRead(notificationId: string, userId: string): Promise<void> {
    await this.notificationRepository.update(
      { id: notificationId, recipient: { id: userId } },
      { isRead: true }
    );
  }

  async markAllNotificationsAsRead(userId: string): Promise<void> {
    await this.notificationRepository.update(
      { recipient: { id: userId }, isRead: false },
      { isRead: true }
    );
  }

  async getUnreadNotificationCount(userId: string): Promise<number> {
    return this.notificationRepository.count({
      where: { recipient: { id: userId }, isRead: false },
    });
  }

  // Cleanup old notifications (run daily at 2 AM)
  @Cron(CronExpression.EVERY_DAY_AT_2AM)
  async cleanupOldNotifications() {
    try {
      const thirtyDaysAgo = new Date();
      thirtyDaysAgo.setDate(thirtyDaysAgo.getDate() - 30);

      const result = await this.notificationRepository
        .createQueryBuilder()
        .delete()
        .where('createdAt < :date', { date: thirtyDaysAgo })
        .execute();

      this.logger.log(`Cleaned up ${result.affected} old notifications`);
    } catch (error) {
      this.logger.error('Failed to cleanup old notifications', error);
    }
  }

  // Retry failed notifications (run every 10 minutes)
  @Cron(CronExpression.EVERY_10_MINUTES)
  async retryFailedNotifications() {
    try {
      const failedNotifications = await this.notificationRepository.find({
        where: { isSent: false },
        relations: ['recipient'],
        take: 50, // Limit to avoid overwhelming the system
      });

      for (const notification of failedNotifications) {
        await this.sendPushNotification(notification);
      }

      if (failedNotifications.length > 0) {
        this.logger.log(`Retried ${failedNotifications.length} failed notifications`);
      }
    } catch (error) {
      this.logger.error('Failed to retry notifications', error);
    }
  }
}