import { Injectable, CanActivate, ExecutionContext, ForbiddenException, BadRequestException } from '@nestjs/common';
import { Reflector } from '@nestjs/core';
import { OrganizationService } from '../organization.service';

@Injectable()
export class OrganizationGuard implements CanActivate {
  constructor(
    private reflector: Reflector,
    private organizationService: OrganizationService,
  ) {}

  async canActivate(context: ExecutionContext): Promise<boolean> {
    const request = context.switchToHttp().getRequest();
    const user = request.user;

    if (!user) {
      throw new ForbiddenException('User not authenticated');
    }

    // Try to get organization ID from various sources
    let orgId = request.params?.orgId || 
                request.body?.organizationId || 
                request.query?.orgId ||
                request.headers['x-organization-id'];

    // If no orgId provided, try to get user's active organization
    if (!orgId) {
      const activeOrg = await this.organizationService.getUserActiveOrganization(user.id);
      if (!activeOrg) {
        throw new BadRequestException('No organization context provided and no active organization found');
      }
      orgId = activeOrg.id;
    }

    // Check if user is a member of the organization
    const isMember = await this.organizationService.isUserMember(orgId, user.id);
    if (!isMember) {
      throw new ForbiddenException('You are not a member of this organization');
    }

    // Get member details and add to request
    const member = await this.organizationService.getMember(orgId, user.id);
    request.organization = member.organization;
    request.organizationMember = member;

    return true;
  }
}