import {
  Controller,
  Get,
  Post,
  Put,
  Delete,
  Body,
  Param,
  Query,
  UseGuards,
  Request,
} from '@nestjs/common';
import { ApiTags, ApiOperation, ApiResponse, ApiBearerAuth } from '@nestjs/swagger';
import { OrganizationService } from './organization.service';
import { CreateOrganizationDto } from './dto/create-organization.dto';
import { UpdateOrganizationDto } from './dto/update-organization.dto';
import { InviteUserDto } from './dto/invite-user.dto';
import { JwtAuthGuard } from '../auth/guards/jwt-auth.guard';
import { OrganizationRole } from '../database/entities/organization-member.entity';

@ApiTags('Organizations')
@Controller('organizations')
@UseGuards(JwtAuthGuard)
@ApiBearerAuth()
export class OrganizationController {
  constructor(private readonly organizationService: OrganizationService) {}

  @Post()
  @ApiOperation({ summary: 'Create a new organization' })
  @ApiResponse({ status: 201, description: 'Organization created successfully' })
  async createOrganization(@Request() req, @Body() createOrgDto: CreateOrganizationDto) {
    return this.organizationService.createOrganization(createOrgDto, req.user.id);
  }

  @Get()
  @ApiOperation({ summary: 'Get user organizations' })
  @ApiResponse({ status: 200, description: 'Organizations retrieved successfully' })
  async getUserOrganizations(@Request() req) {
    return this.organizationService.getUserOrganizations(req.user.id);
  }

  @Get('active')
  @ApiOperation({ summary: 'Get user active organization' })
  @ApiResponse({ status: 200, description: 'Active organization retrieved successfully' })
  async getUserActiveOrganization(@Request() req) {
    return this.organizationService.getUserActiveOrganization(req.user.id);
  }

  @Post(':orgId/set-active')
  @ApiOperation({ summary: 'Set active organization for user' })
  @ApiResponse({ status: 200, description: 'Active organization updated successfully' })
  async setUserActiveOrganization(@Request() req, @Param('orgId') orgId: string) {
    await this.organizationService.setUserActiveOrganization(req.user.id, orgId);
    return { message: 'Active organization updated successfully' };
  }

  @Get(':orgId')
  @ApiOperation({ summary: 'Get organization details' })
  @ApiResponse({ status: 200, description: 'Organization retrieved successfully' })
  async getOrganization(@Request() req, @Param('orgId') orgId: string) {
    return this.organizationService.getOrganization(orgId, req.user.id);
  }

  @Put(':orgId')
  @ApiOperation({ summary: 'Update organization' })
  @ApiResponse({ status: 200, description: 'Organization updated successfully' })
  async updateOrganization(
    @Request() req,
    @Param('orgId') orgId: string,
    @Body() updateOrgDto: UpdateOrganizationDto,
  ) {
    return this.organizationService.updateOrganization(orgId, updateOrgDto, req.user.id);
  }

  @Get(':orgId/members')
  @ApiOperation({ summary: 'Get organization members' })
  @ApiResponse({ status: 200, description: 'Members retrieved successfully' })
  async getOrganizationMembers(@Request() req, @Param('orgId') orgId: string) {
    return this.organizationService.getOrganizationMembers(orgId, req.user.id);
  }

  @Post(':orgId/invite')
  @ApiOperation({ summary: 'Invite user to organization' })
  @ApiResponse({ status: 201, description: 'User invited successfully' })
  async inviteUser(
    @Request() req,
    @Param('orgId') orgId: string,
    @Body() inviteDto: InviteUserDto,
  ) {
    return this.organizationService.inviteUser(orgId, inviteDto, req.user.id);
  }

  @Post('accept-invite/:token')
  @ApiOperation({ summary: 'Accept organization invitation' })
  @ApiResponse({ status: 200, description: 'Invitation accepted successfully' })
  async acceptInvite(@Request() req, @Param('token') token: string) {
    return this.organizationService.acceptInvite(token, req.user.id);
  }

  @Delete(':orgId/members/:memberId')
  @ApiOperation({ summary: 'Remove member from organization' })
  @ApiResponse({ status: 200, description: 'Member removed successfully' })
  async removeMember(
    @Request() req,
    @Param('orgId') orgId: string,
    @Param('memberId') memberId: string,
  ) {
    await this.organizationService.removeMember(orgId, memberId, req.user.id);
    return { message: 'Member removed successfully' };
  }

  @Put(':orgId/members/:memberId/role')
  @ApiOperation({ summary: 'Update member role' })
  @ApiResponse({ status: 200, description: 'Member role updated successfully' })
  async updateMemberRole(
    @Request() req,
    @Param('orgId') orgId: string,
    @Param('memberId') memberId: string,
    @Body('role') role: OrganizationRole,
  ) {
    return this.organizationService.updateMemberRole(orgId, memberId, role, req.user.id);
  }

  @Post(':orgId/leave')
  @ApiOperation({ summary: 'Leave organization' })
  @ApiResponse({ status: 200, description: 'Left organization successfully' })
  async leaveOrganization(@Request() req, @Param('orgId') orgId: string) {
    await this.organizationService.leaveOrganization(orgId, req.user.id);
    return { message: 'Left organization successfully' };
  }
}