import { Injectable, NotFoundException, ForbiddenException, ConflictException, BadRequestException } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';
import { v4 as uuidv4 } from 'uuid';
import { Organization, OrganizationStatus } from '../database/entities/organization.entity';
import { OrganizationMember, OrganizationRole, MemberStatus } from '../database/entities/organization-member.entity';
import { OrganizationInvite, InviteStatus } from '../database/entities/organization-invite.entity';
import { User } from '../database/entities/user.entity';
import { NotificationService } from '../notification/notification.service';
import { CreateOrganizationDto } from './dto/create-organization.dto';
import { UpdateOrganizationDto } from './dto/update-organization.dto';
import { InviteUserDto } from './dto/invite-user.dto';

@Injectable()
export class OrganizationService {
  constructor(
    @InjectRepository(Organization)
    private organizationRepository: Repository<Organization>,
    @InjectRepository(OrganizationMember)
    private memberRepository: Repository<OrganizationMember>,
    @InjectRepository(OrganizationInvite)
    private inviteRepository: Repository<OrganizationInvite>,
    @InjectRepository(User)
    private userRepository: Repository<User>,
    private notificationService: NotificationService,
  ) {}

  async createOrganization(createOrgDto: CreateOrganizationDto, ownerId: string): Promise<Organization> {
    const owner = await this.userRepository.findOne({ where: { id: ownerId } });
    if (!owner) {
      throw new NotFoundException('Owner not found');
    }

    // Check if organization name or slug already exists
    const existingOrg = await this.organizationRepository.findOne({
      where: [
        { name: createOrgDto.name },
        { slug: createOrgDto.slug },
      ],
    });

    if (existingOrg) {
      throw new ConflictException('Organization name or slug already exists');
    }

    // Create organization
    const organization = this.organizationRepository.create({
      ...createOrgDto,
      owner,
      settings: {
        allowPublicSignup: false,
        requireInviteApproval: true,
        maxMembers: 50,
        allowFileUploads: true,
        maxFileSize: 100 * 1024 * 1024, // 100MB
        enableEncryption: true,
        customBranding: false,
        ...createOrgDto.settings,
      },
      limits: {
        maxUsers: 50,
        maxChatRooms: 100,
        maxFileStorage: 10 * 1024 * 1024 * 1024, // 10GB
        maxMonthlyMessages: 10000,
        ...createOrgDto.limits,
      },
    });

    const savedOrg = await this.organizationRepository.save(organization);

    // Add owner as member
    const ownerMember = this.memberRepository.create({
      organization: savedOrg,
      user: owner,
      role: OrganizationRole.OWNER,
      status: MemberStatus.ACTIVE,
      joinedAt: new Date(),
      permissions: {
        canCreateChannels: true,
        canDeleteMessages: true,
        canKickMembers: true,
        canInviteUsers: true,
        canManageRoles: true,
        canAccessAnalytics: true,
      },
    });

    await this.memberRepository.save(ownerMember);

    return savedOrg;
  }

  async getOrganization(orgId: string, userId: string): Promise<Organization> {
    const organization = await this.organizationRepository.findOne({
      where: { id: orgId },
      relations: ['owner', 'members', 'members.user'],
    });

    if (!organization) {
      throw new NotFoundException('Organization not found');
    }

    // Check if user is a member
    const isMember = await this.isUserMember(orgId, userId);
    if (!isMember) {
      throw new ForbiddenException('You are not a member of this organization');
    }

    return organization;
  }

  async getUserOrganizations(userId: string): Promise<Organization[]> {
    const memberships = await this.memberRepository.find({
      where: { user: { id: userId }, status: MemberStatus.ACTIVE },
      relations: ['organization', 'organization.owner'],
      order: { joinedAt: 'DESC' },
    });

    return memberships.map(membership => membership.organization);
  }

  async updateOrganization(
    orgId: string,
    updateOrgDto: UpdateOrganizationDto,
    userId: string,
  ): Promise<Organization> {
    const organization = await this.getOrganization(orgId, userId);
    
    // Check if user has admin permissions
    const member = await this.getMember(orgId, userId);
    if (member.role !== OrganizationRole.OWNER && member.role !== OrganizationRole.ADMIN) {
      throw new ForbiddenException('You do not have permission to update this organization');
    }

    // If updating slug, check for conflicts
    if (updateOrgDto.slug && updateOrgDto.slug !== organization.slug) {
      const existingOrg = await this.organizationRepository.findOne({
        where: { slug: updateOrgDto.slug },
      });
      if (existingOrg) {
        throw new ConflictException('Slug already exists');
      }
    }

    await this.organizationRepository.update(orgId, updateOrgDto);
    return this.getOrganization(orgId, userId);
  }

  async inviteUser(orgId: string, inviteDto: InviteUserDto, inviterId: string): Promise<OrganizationInvite> {
    const organization = await this.getOrganization(orgId, inviterId);
    const inviter = await this.userRepository.findOne({ where: { id: inviterId } });

    // Check if inviter has permission to invite users
    const inviterMember = await this.getMember(orgId, inviterId);
    if (!inviterMember.permissions?.canInviteUsers) {
      throw new ForbiddenException('You do not have permission to invite users');
    }

    // Check if user is already a member
    const existingMember = await this.memberRepository.findOne({
      where: { organization: { id: orgId }, user: { email: inviteDto.email } },
    });

    if (existingMember) {
      throw new ConflictException('User is already a member of this organization');
    }

    // Check if there's already a pending invite
    const existingInvite = await this.inviteRepository.findOne({
      where: { 
        organization: { id: orgId }, 
        email: inviteDto.email, 
        status: InviteStatus.PENDING 
      },
    });

    if (existingInvite) {
      throw new ConflictException('User already has a pending invite');
    }

    // Create invite
    const invite = this.inviteRepository.create({
      organization,
      email: inviteDto.email,
      role: inviteDto.role || OrganizationRole.MEMBER,
      message: inviteDto.message,
      token: uuidv4(),
      expiresAt: new Date(Date.now() + 7 * 24 * 60 * 60 * 1000), // 7 days
      invitedBy: inviter,
    });

    const savedInvite = await this.inviteRepository.save(invite);

    // Send notification if user exists
    const existingUser = await this.userRepository.findOne({ where: { email: inviteDto.email } });
    if (existingUser) {
      await this.notificationService.createNotification(
        existingUser.id,
        'organization_invite' as any,
        `Invitation to join ${organization.name}`,
        `${inviter.username} invited you to join ${organization.name}`,
        {
          organizationId: orgId,
          inviteId: savedInvite.id,
          inviterName: inviter.username,
        },
      );
    }

    return savedInvite;
  }

  async acceptInvite(token: string, userId: string): Promise<OrganizationMember> {
    const invite = await this.inviteRepository.findOne({
      where: { token, status: InviteStatus.PENDING },
      relations: ['organization'],
    });

    if (!invite) {
      throw new NotFoundException('Invite not found or already processed');
    }

    if (invite.expiresAt < new Date()) {
      await this.inviteRepository.update(invite.id, { status: InviteStatus.EXPIRED });
      throw new BadRequestException('Invite has expired');
    }

    const user = await this.userRepository.findOne({ where: { id: userId } });
    if (!user || user.email !== invite.email) {
      throw new ForbiddenException('You are not authorized to accept this invite');
    }

    // Check organization limits
    const memberCount = await this.memberRepository.count({
      where: { organization: { id: invite.organization.id }, status: MemberStatus.ACTIVE },
    });

    if (memberCount >= invite.organization.limits?.maxUsers) {
      throw new BadRequestException('Organization has reached maximum member limit');
    }

    // Create membership
    const member = this.memberRepository.create({
      organization: invite.organization,
      user,
      role: invite.role,
      status: MemberStatus.ACTIVE,
      joinedAt: new Date(),
      invitedBy: invite.invitedBy,
      permissions: this.getDefaultPermissions(invite.role),
    });

    const savedMember = await this.memberRepository.save(member);

    // Update invite status
    await this.inviteRepository.update(invite.id, { 
      status: InviteStatus.ACCEPTED,
      acceptedBy: user,
    });

    return savedMember;
  }

  async removeMember(orgId: string, memberIdToRemove: string, removerId: string): Promise<void> {
    const organization = await this.getOrganization(orgId, removerId);
    const removerMember = await this.getMember(orgId, removerId);

    // Check if remover has permission
    if (!removerMember.permissions?.canKickMembers && removerMember.role !== OrganizationRole.OWNER) {
      throw new ForbiddenException('You do not have permission to remove members');
    }

    const memberToRemove = await this.memberRepository.findOne({
      where: { organization: { id: orgId }, user: { id: memberIdToRemove } },
      relations: ['user'],
    });

    if (!memberToRemove) {
      throw new NotFoundException('Member not found');
    }

    // Cannot remove owner
    if (memberToRemove.role === OrganizationRole.OWNER) {
      throw new ForbiddenException('Cannot remove organization owner');
    }

    // Non-owners cannot remove admins
    if (memberToRemove.role === OrganizationRole.ADMIN && removerMember.role !== OrganizationRole.OWNER) {
      throw new ForbiddenException('Only owners can remove admins');
    }

    await this.memberRepository.update(memberToRemove.id, { status: MemberStatus.LEFT });
  }

  async updateMemberRole(
    orgId: string,
    memberIdToUpdate: string,
    newRole: OrganizationRole,
    updaterId: string,
  ): Promise<OrganizationMember> {
    const updaterMember = await this.getMember(orgId, updaterId);

    // Check if updater has permission
    if (!updaterMember.permissions?.canManageRoles && updaterMember.role !== OrganizationRole.OWNER) {
      throw new ForbiddenException('You do not have permission to manage roles');
    }

    const memberToUpdate = await this.memberRepository.findOne({
      where: { organization: { id: orgId }, user: { id: memberIdToUpdate } },
      relations: ['user'],
    });

    if (!memberToUpdate) {
      throw new NotFoundException('Member not found');
    }

    // Cannot change owner role
    if (memberToUpdate.role === OrganizationRole.OWNER) {
      throw new ForbiddenException('Cannot change owner role');
    }

    // Only owners can assign admin role
    if (newRole === OrganizationRole.ADMIN && updaterMember.role !== OrganizationRole.OWNER) {
      throw new ForbiddenException('Only owners can assign admin role');
    }

    await this.memberRepository.update(memberToUpdate.id, {
      role: newRole,
      permissions: this.getDefaultPermissions(newRole),
    });

    return this.memberRepository.findOne({
      where: { id: memberToUpdate.id },
      relations: ['user', 'organization'],
    });
  }

  async getOrganizationMembers(orgId: string, userId: string): Promise<OrganizationMember[]> {
    await this.getOrganization(orgId, userId); // Check access

    return this.memberRepository.find({
      where: { organization: { id: orgId }, status: MemberStatus.ACTIVE },
      relations: ['user', 'invitedBy'],
      order: { joinedAt: 'ASC' },
    });
  }

  async leaveOrganization(orgId: string, userId: string): Promise<void> {
    const member = await this.getMember(orgId, userId);

    if (member.role === OrganizationRole.OWNER) {
      throw new ForbiddenException('Organization owner cannot leave. Transfer ownership first.');
    }

    await this.memberRepository.update(member.id, { status: MemberStatus.LEFT });
  }

  async isUserMember(orgId: string, userId: string): Promise<boolean> {
    const member = await this.memberRepository.findOne({
      where: {
        organization: { id: orgId },
        user: { id: userId },
        status: MemberStatus.ACTIVE,
      },
    });

    return !!member;
  }

  async getMember(orgId: string, userId: string): Promise<OrganizationMember> {
    const member = await this.memberRepository.findOne({
      where: {
        organization: { id: orgId },
        user: { id: userId },
        status: MemberStatus.ACTIVE,
      },
      relations: ['user', 'organization'],
    });

    if (!member) {
      throw new NotFoundException('You are not a member of this organization');
    }

    return member;
  }

  async getUserActiveOrganization(userId: string): Promise<Organization | null> {
    const membership = await this.memberRepository.findOne({
      where: { user: { id: userId }, status: MemberStatus.ACTIVE },
      relations: ['organization'],
      order: { lastActiveAt: 'DESC' },
    });

    return membership?.organization || null;
  }

  async setUserActiveOrganization(userId: string, orgId: string): Promise<void> {
    const member = await this.getMember(orgId, userId);
    await this.memberRepository.update(member.id, { lastActiveAt: new Date() });
  }

  private getDefaultPermissions(role: OrganizationRole): any {
    switch (role) {
      case OrganizationRole.OWNER:
        return {
          canCreateChannels: true,
          canDeleteMessages: true,
          canKickMembers: true,
          canInviteUsers: true,
          canManageRoles: true,
          canAccessAnalytics: true,
        };
      case OrganizationRole.ADMIN:
        return {
          canCreateChannels: true,
          canDeleteMessages: true,
          canKickMembers: true,
          canInviteUsers: true,
          canManageRoles: false,
          canAccessAnalytics: true,
        };
      case OrganizationRole.MODERATOR:
        return {
          canCreateChannels: true,
          canDeleteMessages: true,
          canKickMembers: false,
          canInviteUsers: true,
          canManageRoles: false,
          canAccessAnalytics: false,
        };
      case OrganizationRole.MEMBER:
      default:
        return {
          canCreateChannels: false,
          canDeleteMessages: false,
          canKickMembers: false,
          canInviteUsers: false,
          canManageRoles: false,
          canAccessAnalytics: false,
        };
    }
  }
}